(*)___________________________________________________________________

  Project:

      MediaTag
      ________

      (C) 2004 Maxim Proskurnya,
               max@axofiber.org.ru

  Component:  id3
  Comments :  Implementation of the id3v1.1 tags read-write
              operations in mp3 media files.
  ____________________________________________________________________

  03.XI.2004                                                     00.00
   ___________________________________________________________________
(*)

unit id3;


interface


type
  TID3FileStatus = ( ifsNOTAGS, ifsNOFILE, ifsID3v1, ifsID3v2 );

  TID3Parser = class
  public
    procedure CheckFile ( FileName: string; var ID3FileStatus: TID3FileStatus );
    procedure ReadID3v1Attributes  ( FileName: string; var Title, Artist, Album, Year, Comment, TrackNumber, Genre: string );
    procedure WriteID3v1Attributes ( FileName: string; Title, Artist, Album, Year, Comment, TrackNumber, Genre: string );
  end;

  PID3Tag = ^TID3Tag;
  TID3Tag = record
    _dummy : char;
  end;

  PID3Frame = ^TID3Frame;
  TID3Frame = record
    _dummy : char;
  end;

var
  Genres : array [0..125] of string;
  

implementation


uses
  Sysutils;

  procedure TID3Parser.CheckFile ( FileName: string; var ID3FileStatus: TID3FileStatus );
  var
    c, c2, c3 : char;
    MediaFile : File of char;
    MediaFileSize : Longint;
  begin
    if FileExists ( FileName ) then
    begin
      ID3FileStatus := ifsNOTAGS;
      AssignFile ( MediaFile, FileName );
      Reset ( MediaFile );
      MediaFileSize :=  FileSize ( MediaFile );
      if 128 < MediaFileSize then
      begin
        Seek ( MediaFile, MediaFileSize - 128 );
        Read ( MediaFile, c, c2, c3 );
        if ( uppercase ( c + c2 + c3 ) = 'TAG' ) then
          ID3FileStatus := ifsID3v1;
      end;
      CloseFile ( MediaFile );
    end
    else
      ID3FileStatus := ifsNOFILE;
  end;



  procedure TID3Parser.ReadID3v1Attributes ( FileName: string; var Title, Artist, Album, Year, Comment, TrackNumber, Genre: string );
  var
    c : char;
    i : integer;
    MediaFile : File of char;
    MediaFileSize : Longint;

    procedure FixID3v1Tag ( var Tag: string );
    var
      n : Integer;
    begin
      n := Length ( Tag );
      while ( 0 < n ) and ( Tag[n] <= ' ' ) do Dec ( n );
      SetLength ( Tag, n );
    end;

  begin
    if FileExists ( FileName ) then
    begin
      AssignFile ( MediaFile, FileName );
      Reset ( MediaFile );
      MediaFileSize :=  FileSize ( MediaFile );
      if 128 < MediaFileSize then
      begin
        Seek ( MediaFile, MediaFileSize - 125 );
        Title  := '';
        Artist := '';
        Album  := '';
        Year   := '';
        Comment:= '';
        Genre  := '';
        for i := 1 to 30 do
        begin
          Read ( MediaFile, c );
          Title := Title + c;
        end;
        for i := 1 to 30 do
        begin
          Read ( MediaFile, c );
          Artist := Artist + c;
        end;
        for i := 1 to 30 do
        begin
          Read ( MediaFile, c );
          Album := Album + c;
        end;
        for i := 1 to 4 do
        begin
          Read ( MediaFile, c );
          Year := Year + c;
        end;
        for i := 1 to 29 do
        begin
          Read ( MediaFile, c );
          Comment := Comment + c;
        end;
        Read ( MediaFile, c );
        TrackNumber := IntToStr ( ord ( c ) );
        Read ( MediaFile, c );
        FixID3v1Tag ( Title );
        FixID3v1Tag ( Artist );
        FixID3v1Tag ( Album );
        FixID3v1Tag ( Year );
        FixID3v1Tag ( Comment );
        Genre := Genres [ ord ( c ) ];
      end;
      CloseFile ( MediaFile );
    end
{
B 30 3-32 
 30 33-62 
D 30 63-92 
E 4 93-96 
F 30 97-125 
G 1 126  
H 1 127 
}
  end;

  procedure TID3Parser.WriteID3v1Attributes ( FileName: string; Title, Artist, Album, Year, Comment, TrackNumber, Genre: string );
  var
    ID3FileStatus: TID3FileStatus;
    MediaFile : File of char;
    MediaFileSize : Integer;
    c : char;

    procedure WriteBuf ( Buf: string; Size: Integer );
    var
      L, i : Integer;
    begin
      L := Length ( Buf );
      if Size < L then
      begin
        SetLength ( Buf, Size );
        L := Size;
      end;
      for i := 1 to L do
        Write ( MediaFile, Buf[i] );
      c := #0;
      for i := L + 1 to Size do
        Write ( MediaFile, c );
    end;

    function FindGenre ( GenreName: string ): Integer;
    var
      i : Integer;
    begin
      i := 0;
      while ( i < 125 ) and ( Genres[i] <> GenreName ) do Inc ( i );
      if i <= 125 then
        Result := i
      else
        Result := 12
    end;

  begin
    CheckFile ( FileName, ID3FileStatus );
    if ID3FileStatus <> ifsNOFILE then
    begin
      AssignFile ( MediaFile, FileName );
      Reset ( MediaFile );
      MediaFileSize :=  FileSize ( MediaFile );
      if ID3FileStatus = ifsNOTAGS then
      begin
        Seek ( MediaFile, MediaFileSize );
        WriteBuf ( 'TAG', 3 );
      end
      else
        Seek ( MediaFile, MediaFileSize - 125 );
      WriteBuf ( Title,   30 );
      WriteBuf ( Artist,  30 );
      WriteBuf ( Album,   30 );
      WriteBuf ( Year,    4 );
      WriteBuf ( Comment, 29 );
      c := chr ( StrToInt ( TrackNumber ) );
      Write    ( MediaFile, c );
      c := chr ( FindGenre ( Genre ) );
      Write    ( MediaFile, c );
      CloseFile ( MediaFile );
    end;
  end;


initialization
  Genres[0]   := 'Blues';
  Genres[1]   := 'Classic Rock';
  Genres[2]   := 'Country';
  Genres[3]   := 'Dance';
  Genres[4]   := 'Disco';
  Genres[5]   := 'Funk';
  Genres[6]   := 'Grunge';
  Genres[7]   := 'Hip-Hop';
  Genres[8]   := 'Jazz';
  Genres[9]   := 'Metal';
  Genres[10]  := 'New Age';
  Genres[11]  := 'Oldies';
  Genres[12]  := 'Other';
  Genres[13]  := 'Pop';
  Genres[14]  := 'R&B';
  Genres[15]  := 'Rap';
  Genres[16]  := 'Reggae';
  Genres[17]  := 'Rock';
  Genres[18]  := 'Techno';
  Genres[19]  := 'Industrial';
  Genres[20]  := 'Alternative';
  Genres[21]  := 'Ska';
  Genres[22]  := 'Death Metal';
  Genres[23]  := 'Pranks';
  Genres[24]  := 'Soundtrack';
  Genres[25]  := 'Euro-Techno';
  Genres[26]  := 'Ambient';
  Genres[27]  := 'Trip-Hop';
  Genres[28]  := 'Vocal';
  Genres[29]  := 'Jazz+Funk';
  Genres[30]  := 'Fusion';
  Genres[31]  := 'Trance';
  Genres[32]  := 'Classical';
  Genres[33]  := 'Instrumental';
  Genres[34]  := 'Acid';
  Genres[35]  := 'House';
  Genres[36]  := 'Game';
  Genres[37]  := 'Sound Clip';
  Genres[38]  := 'Gospel';
  Genres[39]  := 'Noise';
  Genres[40]  := 'AlternRock';
  Genres[41]  := 'Bass';
  Genres[42]  := 'Soul';
  Genres[43]  := 'Punk';
  Genres[44]  := 'Space';
  Genres[45]  := 'Meditative';
  Genres[46]  := 'Instrumental Pop';
  Genres[47]  := 'Instrumental Rock';
  Genres[48]  := 'Ethnic';
  Genres[49]  := 'Gothic';
  Genres[50]  := 'Darkwave';
  Genres[51]  := 'Techno-Industrial';
  Genres[52]  := 'Electronic';
  Genres[53]  := 'Pop-Folk';
  Genres[54]  := 'Eurodance';
  Genres[55]  := 'Dream';
  Genres[56]  := 'Southern Rock';
  Genres[57]  := 'Comedy';
  Genres[58]  := 'Cult';
  Genres[59]  := 'Gangsta';
  Genres[60]  := 'Top 40';
  Genres[61]  := 'Christian Rap';
  Genres[62]  := 'Pop/Funk';
  Genres[63]  := 'Jungle';
  Genres[64]  := 'Native American';
  Genres[65]  := 'Cabaret';
  Genres[66]  := 'New Wave';
  Genres[67]  := 'Psychadelic';
  Genres[68]  := 'Rave';
  Genres[69]  := 'Showtunes';
  Genres[70]  := 'Trailer';
  Genres[71]  := 'Lo-Fi';
  Genres[72]  := 'Tribal';
  Genres[73]  := 'Acid Punk';
  Genres[74]  := 'Acid Jazz';
  Genres[75]  := 'Polka';
  Genres[76]  := 'Retro';
  Genres[77]  := 'Musical';
  Genres[78]  := 'Rock & Roll';
  Genres[79]  := 'Hard Rock';
  Genres[70]  := 'Folk';
  Genres[81]  := 'Folk-Rock';
  Genres[82]  := 'National Folk';
  Genres[83]  := 'Swing';
  Genres[84]  := 'Fast Fusion';
  Genres[85]  := 'Bebob';
  Genres[86]  := 'Latin';
  Genres[87]  := 'Revival';
  Genres[88]  := 'Celtic';
  Genres[89]  := 'Bluegrass';
  Genres[90]  := 'Avantgarde';
  Genres[91]  := 'Gothic Rock';
  Genres[92]  := 'Progressive Rock';
  Genres[93]  := 'Psychedelic Rock';
  Genres[94]  := 'Symphonic Rock';
  Genres[95]  := 'Slow Rock';
  Genres[96]  := 'Big Band';
  Genres[97]  := 'Chorus';
  Genres[98]  := 'Easy Listening';
  Genres[99]  := 'Acoustic';
  Genres[100] := 'Humour';
  Genres[101] := 'Speech';
  Genres[102] := 'Chanson';
  Genres[103] := 'Opera';
  Genres[104] := 'Chamber Music';
  Genres[105] := 'Sonata';
  Genres[106] := 'Symphony';
  Genres[107] := 'Booty Brass';
  Genres[108] := 'Primus';
  Genres[109] := 'Porn Groove';
  Genres[110] := 'Satire';
  Genres[111] := 'Slow Jam';
  Genres[112] := 'Club';
  Genres[113] := 'Tango';
  Genres[114] := 'Samba';
  Genres[115] := 'Folklore';
  Genres[116] := 'Ballad';
  Genres[117] := 'Poweer Ballad';
  Genres[118] := 'Rhytmic Soul';
  Genres[119] := 'Freestyle';
  Genres[120] := 'Duet';
  Genres[121] := 'Punk Rock';
  Genres[122] := 'Drum Solo';
  Genres[123] := 'A Capela';
  Genres[124] := 'Euro-House';
  Genres[125] := 'Dance Hall';

end.



